use std::sync::Arc;
use crate::canvas::managed::handles::{CanvasImageHandle, CanvasTextureHandle};
use crate::canvas::canvas_frame::Drawable;
use crate::util::vertex::{VertexTypeContainer, ImageVertex3D};
use crate::util::tr_event::{TrUIEvent, TrEvent};

pub struct CompuSprite {

    pub verts: VertexTypeContainer,

    position: (f32, f32),
    size: (f32, f32),
    color: (f32, f32, f32, f32),

}

impl CompuSprite {
    pub fn new(position: (f32, f32),
               size: (f32, f32),
               depth: u32,
               image_size: (f32, f32),
               image_handle: Arc<CanvasImageHandle>) -> CompuSprite {

        let normalized_depth = (depth as f32 / 255.0);

        let verts = vec![
            ImageVertex3D {
                v_position: [position.0, position.1, normalized_depth],                   // top left
                ti_position: [-0.0, -0.0] },
            ImageVertex3D {
                v_position: [position.0, position.1 + size.1, normalized_depth],          // bottom left
                ti_position: [-0.0, image_size.1] },
            ImageVertex3D {
                v_position: [position.0 + size.0, position.1 + size.1, normalized_depth], // bottom right
                ti_position: [image_size.0, image_size.1] },
            ImageVertex3D {
                v_position: [position.0, position.1, normalized_depth],                   // top left
                ti_position: [-0.0, -0.0] },
            ImageVertex3D {
                v_position: [position.0 + size.0, position.1 + size.1, normalized_depth], // bottom right
                ti_position: [image_size.0, image_size.1] },
            ImageVertex3D {
                v_position: [position.0 + size.0, position.1, normalized_depth],          // top right
                ti_position: [image_size.0, -0.0] },
        ];

        CompuSprite {
            verts: VertexTypeContainer::ImageType(verts, image_handle.clone()),
            position: position,
            size: size,
            color: (0.0, 0.0, 0.0, 0.0),
        }
    }
}

impl Drawable for CompuSprite {
    fn render(&self,
              window_size: (u32, u32),
              position: (f32, f32),
              rotation: f32,
              size: (f32, f32),
              depth: f32,
    ) -> Vec<VertexTypeContainer> {
        vec![self.verts.clone()]
    }

    // fn update<T>(&self, delta_time: f32) -> Vec<TrUIEvent<T>> {
    //     Vec::new()
    // }
    //
    // fn notify<Y, T>(&self, tr_event: Vec<TrEvent<Y>>, ui_events: Vec<TrUIEvent<T>>) -> Vec<TrUIEvent<T>> {
    //     Vec::new()
    // }
}